@@ -79,9 +79,9 @@ SYMBOL symbols[] = {
   { "AT",		SYM(AT_SYM)},
   { "ATOMIC",		SYM(ATOMIC_SYM)},
   { "AUTHORS",	        SYM(AUTHORS_SYM)},
+  { "AUTO",             SYM(AUTO_SYM)},
   { "AUTO_INCREMENT",	SYM(AUTO_INC)},
   { "AUTOEXTEND_SIZE",	SYM(AUTOEXTEND_SIZE_SYM)},
-  { "AUTO",	        SYM(AUTO_SYM)},
   { "AVG",		SYM(AVG_SYM)},
   { "AVG_ROW_LENGTH",	SYM(AVG_ROW_LENGTH)},
   { "BACKUP",	        SYM(BACKUP_SYM)},
@@ -385,6 +385,8 @@ SYMBOL symbols[] = {
   { "MASTER_SSL_VERIFY_SERVER_CERT", SYM(MASTER_SSL_VERIFY_SERVER_CERT_SYM)},
   { "MASTER_USER",           SYM(MASTER_USER_SYM)},
   { "MASTER_USE_GTID",  SYM(MASTER_USE_GTID_SYM)},
+  { "MASTER_DEMOTE_TO_REPLICA",  SYM(MASTER_DEMOTE_TO_SLAVE_SYM)},
+  { "MASTER_DEMOTE_TO_SLAVE",  SYM(MASTER_DEMOTE_TO_SLAVE_SYM)},
   { "MASTER_HEARTBEAT_PERIOD", SYM(MASTER_HEARTBEAT_PERIOD_SYM)},
   { "MATCH",		SYM(MATCH)},
   { "MAX_CONNECTIONS_PER_HOUR", SYM(MAX_CONNECTIONS_PER_HOUR)},
@@ -427,7 +429,6 @@ SYMBOL symbols[] = {
   { "NCHAR",		SYM(NCHAR_SYM)},
   { "NESTED",		SYM(NESTED_SYM)},
   { "NEVER",		SYM(NEVER_SYM)},
-  { "NEW",              SYM(NEW_SYM)},
   { "NEXT",		SYM(NEXT_SYM)},
   { "NEXTVAL",		SYM(NEXTVAL_SYM)},
   { "NO",		SYM(NO_SYM)},
@@ -617,6 +618,8 @@ SYMBOL symbols[] = {
   { "SQLEXCEPTION",     SYM(SQLEXCEPTION_SYM)},
   { "SQLSTATE",         SYM(SQLSTATE_SYM)},
   { "SQLWARNING",       SYM(SQLWARNING_SYM)},
+  { "SQL_AFTER_GTIDS",  SYM(SQL_AFTER_GTIDS_SYM)},
+  { "SQL_BEFORE_GTIDS",  SYM(SQL_BEFORE_GTIDS_SYM)},
   { "SQL_BIG_RESULT",	SYM(SQL_BIG_RESULT)},
   { "SQL_BUFFER_RESULT", SYM(SQL_BUFFER_RESULT)},
   { "SQL_CACHE",        SYM(SQL_CACHE_SYM)},
@@ -685,7 +688,6 @@ SYMBOL symbols[] = {
   { "TRUE",		SYM(TRUE_SYM)},
   { "TRUNCATE",		SYM(TRUNCATE_SYM)},
   { "TYPE",		SYM(TYPE_SYM)},
-  { "TYPES",		SYM(TYPES_SYM)},
   { "UNBOUNDED",        SYM(UNBOUNDED_SYM)},
   { "UNCOMMITTED",	SYM(UNCOMMITTED_SYM)},
   { "UNDEFINED",	SYM(UNDEFINED_SYM)},
@@ -711,6 +713,7 @@ SYMBOL symbols[] = {
   { "UTC_DATE",         SYM(UTC_DATE_SYM)},
   { "UTC_TIME",         SYM(UTC_TIME_SYM)},
   { "UTC_TIMESTAMP",    SYM(UTC_TIMESTAMP_SYM)},
+  { "VALIDATION",       SYM(VALIDATION_SYM)},
   { "VALUE",		SYM(VALUE_SYM)},
   { "VALUES",		SYM(VALUES)},
   { "VARBINARY",	SYM(VARBINARY)},
@@ -751,7 +754,6 @@ SYMBOL symbols[] = {
 
 SYMBOL sql_functions[] = {
   { "ADDDATE",		SYM(ADDDATE_SYM)},
-  { "ADD_MONTHS",	SYM(ADD_MONTHS_SYM)},
   { "BIT_AND",		SYM(BIT_AND)},
   { "BIT_OR",		SYM(BIT_OR)},
   { "BIT_XOR",		SYM(BIT_XOR)},
@@ -762,8 +764,6 @@ SYMBOL sql_functions[] = {
   { "CURTIME",		SYM(CURTIME)},
   { "DATE_ADD",		SYM(DATE_ADD_INTERVAL)},
   { "DATE_SUB",		SYM(DATE_SUB_INTERVAL)},
-  { "DATE_FORMAT",      SYM(DATE_FORMAT_SYM)},
-  { "DECODE",           SYM(DECODE_MARIADB_SYM)},
   { "DENSE_RANK",       SYM(DENSE_RANK_SYM)},
   { "EXTRACT",		SYM(EXTRACT_SYM)},
   { "FIRST_VALUE",      SYM(FIRST_VALUE_SYM)},
