@@ -78,11 +78,16 @@ struct Vers_part_info : public Sql_alloc
     }
     return false;
   }
-  struct {
+  struct interval_t {
     my_time_t start;
     INTERVAL step;
     enum interval_type type;
     bool is_set() const { return type < INTERVAL_LAST; }
+    bool operator==(const interval_t &rhs) const
+    {
+      /* TODO: equivalent intervals like 1 hour and 60 mins should be considered equal */
+      return start == rhs.start && type == rhs.type && !memcmp(&step, &rhs.step, sizeof(INTERVAL));
+    }
   } interval;
   ulonglong limit;
   bool auto_hist;
@@ -90,6 +95,10 @@ struct Vers_part_info : public Sql_alloc
   partition_element *hist_part;
 };
 
+/*
+  See generate_partition_syntax() for details of how the data is used
+  in partition expression.
+*/
 class partition_info : public DDL_LOG_STATE, public Sql_alloc
 {
 public:
@@ -99,6 +108,10 @@ public:
   List<partition_element> partitions;
   List<partition_element> temp_partitions;
 
+  /*
+    These are mutually exclusive with part_expr/subpart_expr depending on
+    what is specified in partitioning filter: expression or column list.
+  */
   List<const char> part_field_list;
   List<const char> subpart_field_list;
   
@@ -335,12 +348,11 @@ public:
     part_field_list.empty();
     subpart_field_list.empty();
   }
-  ~partition_info() {}
+  ~partition_info() = default;
 
   partition_info *get_clone(THD *thd, bool empty_data_and_index_file= FALSE);
   bool set_named_partition_bitmap(const char *part_name, size_t length);
   bool set_partition_bitmaps(List<String> *partition_names);
-  bool set_partition_bitmaps_from_table(TABLE_LIST *table_list);
   /* Answers the question if subpartitioning is used for a certain table */
   bool is_sub_partitioned()
   {
@@ -427,8 +439,13 @@ public:
     return NULL;
   }
   uint next_part_no(uint new_parts) const;
+
+  int gen_part_type(THD *thd, String *str) const;
 };
 
+void part_type_error(THD *thd, partition_info *work_part_info,
+                     const char *part_type, partition_info *tab_part_info);
+
 uint32 get_next_partition_id_range(struct st_partition_iter* part_iter);
 bool check_partition_dirs(partition_info *part_info);
 bool vers_create_partitions(THD* thd, TABLE_LIST* tl, uint num_parts);
