@@ -99,6 +99,7 @@ public:
   bool send_result_set_row(List<Item> *row_items);
 
   bool store(I_List<i_string> *str_list);
+  bool store(I_List<i_string_pair> *str_list);
   bool store_string_or_null(const char *from, CHARSET_INFO *cs);
   bool store_warning(const char *from, size_t length);
   String *storage_packet() { return packet; }
@@ -198,12 +199,10 @@ class Protocol_text :public Protocol
   StringBuffer<FLOATING_POINT_BUFFER> buffer;
   bool store_numeric_string_aux(const char *from, size_t length);
 public:
-  Protocol_text(THD *thd_arg, ulong prealloc= 0)
-   :Protocol(thd_arg)
-  {
-    if (prealloc)
-      packet->alloc(prealloc);
-  }
+  Protocol_text(THD *thd_arg)
+   :Protocol(thd_arg) {};
+  bool __attribute__((warn_unused_result))
+       allocate(size_t size) { return packet->alloc(size); }
   void prepare_for_resend() override;
   bool store_null() override;
   bool store_tiny(longlong from) override;
@@ -228,9 +227,9 @@ public:
 #ifdef EMBEDDED_LIBRARY
   void remove_last_row() override;
 #endif
-  virtual bool store_field_metadata(const THD *thd, const Send_field &field,
-                                    CHARSET_INFO *charset_for_protocol,
-                                    uint pos);
+  bool store_field_metadata(const THD *thd, const Send_field &field,
+                            CHARSET_INFO *charset_for_protocol,
+                            uint pos);
   bool store_item_metadata(THD *thd, Item *item, uint pos);
   bool store_field_metadata_for_list_fields(const THD *thd, Field *field,
                                             const TABLE_LIST *table_list,
