@@ -18,8 +18,9 @@
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1335  USA */
 
 #include "my_base.h"                            /* ha_rows */
-#include <my_sys.h>                             /* qsort2_cmp */
+#include <my_cmp.h>
 #include "queues.h"
+#include "sql_string.h"
 #include "sql_class.h"
 
 class Field;
@@ -541,14 +542,25 @@ to be fixed later
 
 class Sort_param {
 public:
-  uint rec_length;            // Length of sorted records.
-  uint sort_length;           // Length of sorted columns.
+  // Length of sorted records. ALWAYS equal to sort_length + addon_length
+  uint rec_length;
+  /*
+    Length of what we need to sort: Sorted columns + ref_length if not
+    addon fields are used
+  */
+  uint sort_length;
+  /* Length of the reference to the row (rowid or primary key etc */
   uint ref_length;            // Length of record ref.
+  /* Length of all addon fields. 0 if no addon fields */
   uint addon_length;          // Length of addon_fields
-  uint res_length;            // Length of records in final sorted file/buffer.
+  /*
+    The length of the 'result' we are going to return to the caller for
+    each sort element. Also the length of data in final sorted file/buffer.
+  */
+  uint res_length;
   uint max_keys_per_buffer;   // Max keys / buffer.
   uint min_dupl_count;
-  ha_rows max_rows;           // Select limit, or HA_POS_ERROR if unlimited.
+  ha_rows limit_rows;         // Select limit, or HA_POS_ERROR if unlimited.
   ha_rows examined_rows;      // Number of examined rows.
   TABLE *sort_form;           // For quicker make_sortkey.
   /**
@@ -558,13 +570,15 @@ public:
   Bounds_checked_array<SORT_FIELD> local_sortorder;
   Addon_fields *addon_fields;     // Descriptors for companion fields.
   Sort_keys *sort_keys;
+  ha_rows *accepted_rows;         /* For ROWNUM */
   bool using_pq;
+  bool set_all_read_bits;
 
   uchar *unique_buff;
   bool not_killable;
   String tmp_buffer;
   // The fields below are used only by Unique class.
-  qsort2_cmp compare;
+  qsort_cmp2 compare;
   BUFFPEK_COMPARE_CONTEXT cmp_context;
 
   Sort_param()
@@ -577,8 +591,14 @@ public:
     */
     tmp_buffer.set_charset(&my_charset_bin);
   }
-  void init_for_filesort(uint sortlen, TABLE *table,
-                         ha_rows maxrows, bool sort_positions);
+
+  void init_for_filesort(TABLE *table, Filesort *filesort,
+                         uint sortlen, ha_rows limit_rows_arg);
+  void setup_lengths_and_limit(TABLE *table,
+                               uint sortlen,
+                               uint addon_length,
+                               ha_rows limit_rows_arg);
+  void  (*unpack)(TABLE *);
   /// Enables the packing of addons if possible.
   void try_to_pack_addons(ulong max_length_for_sort_data);
 
@@ -668,7 +688,7 @@ public:
 
   void try_to_pack_sortkeys();
 
-  qsort2_cmp get_compare_function() const
+  qsort_cmp2 get_compare_function() const
   {
     return using_packed_sortkeys() ?
            get_packed_keys_compare_ptr() :
