@@ -406,10 +406,10 @@ public:
   }
 
   // Returns offset to substring or -1
-  int strstr(const Binary_string &search, uint32 offset=0);
-  int strstr(const char *search, uint32 search_length, uint32 offset=0);
+  int strstr(const Binary_string &search, uint32 offset=0) const;
+  int strstr(const char *search, uint32 search_length, uint32 offset=0) const;
   // Returns offset to substring or -1
-  int strrstr(const Binary_string &search, uint32 offset=0);
+  int strrstr(const Binary_string &search, uint32 offset=0) const;
 
   /*
     The following append operations do not extend the strings and in production
@@ -446,6 +446,19 @@ public:
     float8store(Ptr + str_length, *d);
     str_length += 8;
   }
+  /*
+    Append a wide character.
+    The caller must have allocated at least cs->mbmaxlen bytes.
+  */
+  int q_append_wc(my_wc_t wc, CHARSET_INFO *cs)
+  {
+    int mblen;
+    if ((mblen= cs->cset->wc_mb(cs, wc,
+                                (uchar *) end(),
+                                (uchar *) end() + cs->mbmaxlen)) > 0)
+      str_length+= (uint32) mblen;
+    return mblen;
+  }
   void q_append(const char *data, size_t data_len)
   {
     ASSERT_LENGTH(data_len);
@@ -572,7 +585,7 @@ public:
     LEX_CSTRING tmp= {Ptr, str_length};
     return tmp;
   }
-  inline LEX_CSTRING *get_value(LEX_CSTRING *res)
+  inline LEX_CSTRING *get_value(LEX_CSTRING *res) const
   {
     res->str=    Ptr;
     res->length= str_length;
@@ -896,6 +909,8 @@ public:
    :Charset(cs), Binary_string(str, len)
   { }
   String(const String &str) = default;
+  String(String &&str) noexcept
+   :Charset(std::move(str)), Binary_string(std::move(str)){}
 
   void set(String &str,size_t offset,size_t arg_length)
   {
@@ -1052,13 +1067,6 @@ public:
   }
 
   // Append with optional character set conversion from ASCII (e.g. to UCS2)
-  bool append(const LEX_STRING *ls)
-  {
-    DBUG_ASSERT(ls->length < UINT_MAX32 &&
-                ((ls->length == 0 && !ls->str) ||
-                 ls->length == strlen(ls->str)));
-    return append(ls->str, (uint32) ls->length);
-  }
   bool append(const LEX_CSTRING *ls)
   {
     DBUG_ASSERT(ls->length < UINT_MAX32 &&
@@ -1082,8 +1090,6 @@ public:
       (quot && append(quot));
   }
   bool append(const char *s, size_t size);
-  bool append_with_prefill(const char *s, uint32 arg_length,
-			   uint32 full_length, char fill_char);
   bool append_parenthesized(long nr, int radix= 10);
 
   // Append with optional character set conversion from cs to charset()
@@ -1093,6 +1099,31 @@ public:
     return append(s.str, s.length, cs);
   }
 
+  // Append a wide character
+  bool append_wc(my_wc_t wc)
+  {
+    if (reserve(mbmaxlen()))
+      return true;
+    int mblen= q_append_wc(wc, charset());
+    if (mblen > 0)
+      return false;
+    else if (mblen == MY_CS_ILUNI && wc != '?')
+      return q_append_wc('?', charset()) <= 0;
+    return true;
+  }
+
+  // Append a number with zero prefilling
+  bool append_zerofill(uint num, uint width)
+  {
+    static const char zeros[15]= "00000000000000";
+    char intbuff[15];
+    uint length= (uint) (int10_to_str(num, intbuff, 10) - intbuff);
+    if (length < width &&
+        append(zeros, width - length, &my_charset_latin1))
+      return true;
+    return append(intbuff, length, &my_charset_latin1);
+  }
+
   /*
     Append a bitmask in an uint32 with a translation into a
     C-style human readable representation, e.g.:
@@ -1152,6 +1183,43 @@ public:
       print_with_conversion(to, cs);
   }
 
+  static my_wc_t escaped_wc_for_single_quote(my_wc_t ch)
+  {
+    switch (ch) {
+    case '\\':   return '\\';
+    case '\0':   return '0';
+    case '\'':   return '\'';
+    case '\b':   return 'b';
+    case '\t':   return 't';
+    case '\n':   return 'n';
+    case '\r':   return 'r';
+    case '\032': return 'Z';
+    }
+    return 0;
+  }
+
+  // Append for single quote using mb_wc/wc_mb Unicode conversion
+  bool append_for_single_quote_using_mb_wc(const char *str, size_t length,
+                                           CHARSET_INFO *cs);
+
+  // Append for single quote with optional mb_wc/wc_mb conversion
+  bool append_for_single_quote_opt_convert(const char *str,
+                                           size_t length,
+                                           CHARSET_INFO *cs)
+  {
+    return charset() == &my_charset_bin || cs == &my_charset_bin  ||
+           my_charset_same(charset(), cs) ?
+           append_for_single_quote(str, length) :
+           append_for_single_quote_using_mb_wc(str, length, cs);
+  }
+
+  bool append_for_single_quote_opt_convert(const String &str)
+  {
+    return append_for_single_quote_opt_convert(str.ptr(),
+                                               str.length(),
+                                               str.charset());
+  }
+
   bool append_for_single_quote(const char *st, size_t len);
   bool append_for_single_quote(const String *s)
   {
